# -*- coding: utf-8 -*-
from __future__ import unicode_literals


from driverng.utils import (
    ACK,
    NACK,
    STX,
    byte2log,
)
from driverng.protocol.base import BaseProtocol


class SQAVisionProtocol(BaseProtocol):
    def __init__(self, *args, **kwargs):
        super(SQAVisionProtocol, self).__init__(**kwargs)

    def send_trames(self, trames):
        pass

    def build_trames(self, message):
        pass

    def recv_trames(self):
        '''Recoit une liste de trames sur la connexion:
        format de la trame : <STX><index><payload><ETX><CHECKSUM_char1><CHECKSUM_char2><CR><LF>

        <- DATA 1
        -> <ack>
        <- DATA 2
        -> <ack>
        <- DATA 3
        -> <nack>
        <- DATA 3
        -> <ack>

        returns False lors d'une erreur
        returns la liste des messages
        '''

        stx = self.read()
        if not stx or stx[0] != STX:
            self.logger.warning('<stx> expected in %s', byte2log(stx))
            return False
        # ...<index><payload><ETX><CHECKSUM_char1><CHECKSUM_char2><CR><LF>
        data = self.readline(timeout=15)
        if self.check_trame(data):
            self.write(ACK)
            if data[1:3] == b"O|" and len(data.split(b"|")) > 2:
                return self.get_payload(data)
        else:
            self.write(NACK)
            return False

        return False

    def check_trame(self, trame):
        try:
            checksum = int(trame[-4:-2], 16)
            checksum_expected = self.checksum(trame[:-4])
        except ValueError:
            self.logger.warning(
                'Missing checksum for %s',
                byte2log(trame),
            )
            return False
        if checksum != checksum_expected:
            self.logger.warning(
                'Bad checksum for %s: (%02X != %02X)',
                byte2log(trame),
                checksum,
                checksum_expected)
            return False
        return True

    def get_payload(self, trame):
        # <payload><ETX><CHECKSUM_char1><CHECKSUM_char2><CR><LF>
        payload = trame[1:-5]
        return payload

    @staticmethod
    def checksum(data):
        return (sum(bytearray(data)) & 0xFF) % 0x100
